#!/usr/bin/env python
# coding: utf-8

# In[12]:
import math

def draw_snowflake(endpoints, depth):
    '''Draw the complete snowflake.'''
    # Draw the snowflake.
    points = [endpoints[0]]
    for i in range(1, len(endpoints)):
        p1 = endpoints[i - 1]
        p2 = endpoints[i]
        length = math.dist(p1, p2)
        dx = p2[0] - p1[0]
        dy = p2[1] - p1[1]
        theta = math.atan2(dy, dx)

        p1 = draw_koch_curve(points, depth, p1, theta, length)

    return points

def draw_koch_curve(points, depth, p1, theta, length):
    '''
    Recursively draw a snowflake edge starting at p1 in direction theta
    and distance dist. Return the endpoint.
    '''
    if depth == 0:
        # Just move there.
        p2 = (p1[0] + length * math.cos(theta),
              p1[1] + length * math.sin(theta))
        points.append(p2)
        return p2

    # Recursively draw the edge.
    length /= 3
    p1 = draw_koch_curve(points, depth - 1, p1, theta, length)
    theta -= math.radians(60)
    p1 = draw_koch_curve(points, depth - 1, p1, theta, length)
    theta += math.radians(120)
    p1 = draw_koch_curve(points, depth - 1, p1, theta, length)
    theta -= math.radians(60)
    p1 = draw_koch_curve(points, depth - 1, p1, theta, length)

    return p1 # Return the final end point.

# In[18]:
import tkinter as tk
from tkinter import messagebox

class KochCurveApp:
    def __init__(self):
        self.window = tk.Tk()
        self.window.title('koch_snowflake')
        self.window.protocol('WM_DELETE_WINDOW', self.kill_callback)
        self.window.geometry('400x400')

        # Build the UI.
        frame = tk.Frame(self.window)
        frame.pack(side=tk.TOP, anchor=tk.W, padx=3, pady=3)

        label = tk.Label(frame, text='Depth:')
        label.pack(side=tk.LEFT, padx=3)
        self.depth_var = tk.StringVar(value='3')
        entry = tk.Entry(frame, textvariable=self.depth_var, width=3)
        entry.pack(side=tk.LEFT, padx=3)

        button = tk.Button(frame, text='Draw', width=10, command=self.draw)
        button.pack(side=tk.LEFT, padx=10)

        self.canvas = tk.Canvas(self.window, bg='white',
            borderwidth=2, relief=tk.SUNKEN)
        self.canvas.pack(padx=10, pady=(0, 10),
            side=tk.BOTTOM, fill=tk.BOTH, expand=True)

        # If the user presses Enter, draw.
        entry.bind('<Return>', lambda event: self.draw())

        # Display the window.
        entry.focus_force()
        self.window.mainloop()

    def kill_callback(self):
        # Destroy the tkinter window.
        self.window.destroy()

    def draw(self):
        # Delete any previous drawing objects.
        self.canvas.delete(tk.ALL)

        # Get the desired depth.
        try:
            depth = int(self.depth_var.get())
        except:
            messagebox.showinfo('Depth Error', 'Depth must be an integer between 0 and 10.')
            return
        if depth < 0 or depth > 10:
            messagebox.showinfo('Depth Error', 'Depth must be an integer between 0 and 10.')
            return

        # Find the three corners.
        wid = self.canvas.winfo_width()
        hgt = self.canvas.winfo_height()
        height = 0.75 * min(wid, hgt) - 20
        width = height / math.sqrt(3.0) * 2
        y3 = hgt - 10
        y1 = y3 - height
        x3 = hgt / 2
        x1 = x3 - width / 2
        x2 = x1 + width
        endpoints = [
            (x1, y1),
            (x2, y1),
            (x3, y3),
            (x1, y1),
        ]

        # Get the points.
        points = draw_snowflake(endpoints, depth)

        # Draw the polygon.
        self.canvas.create_polygon(points, fill='light blue')
        self.canvas.create_line(points, width=1, fill='black')

# In[19]:
KochCurveApp()
print('Done')
